@@ -2,6 +2,9 @@
 
 "Pythonic access to high energy particle data tables."
 
+## So exec("float('+2/3')") etc. will work
+from __future__ import division
+
 
 C_IN_MM_PER_PS = 0.299792458
 HBAR_IN_GEVPS = 6.58211814E-13
@@ -81,7 +84,7 @@ class ParticleData(object):
 
     @property
     def ctau(self):
-        "Get the particle c tau distance in mm."
+        "Get the particle c*tau0 distance in mm."
         if self.lifetime is None:
             return None
         return self.lifetime * C_IN_MM_PER_PS
@@ -99,20 +102,20 @@ class ParticleData(object):
         return self.ctau * beta * gamma
 
     def __repr__(self):
-        s = "%s: ID=%d, m=%1.2e GeV, 3*q=%d" % (self.name, self.id, self.mass, self.threecharge)
+        s = "%s: ID=%d, m=%1.2g GeV, 3*q=%d" % (self.name, self.id, self.mass, self.threecharge)
         if self.width is not None:
-            s += ", width=%1.2e GeV" % self.width
+            s += ", width=%1.2g GeV" % self.width
         if self.lifetime is not None:
-            s += ", tau=%1.2e ps" % self.lifetime
+            s += ", tau=%1.2g ps" % self.lifetime
         if self.ctau is not None:
-            s += ", ctau=%1.2e mm" % self.ctau
+            s += ", ctau=%1.2g mm" % self.ctau
         return s
 
 
 ## Add PID functions to the ParticleData object as bound methods
 import pypdt.pid as _PID
 for symname in dir(_PID):
-    if not symname.startswith("_") and not symname.startswith("N"):
+    if not symname.startswith("_") and not symname.startswith("N") and symname not in ("charge", "threeCharge"):
         ## Note lambda binding / lexical closure trick with sym=
         f = lambda self, sym=getattr(_PID, symname) : sym(self.id)
         setattr(ParticleData, symname, f)
@@ -122,9 +125,10 @@ class ParticleDataTable(object):
     """
     Wrapper object for a whole database of ParticleData objects, indexed by their PDG ID codes.
 
-    By default, the input database file will be the particle.tbl file from the most recent HepPDT
-    version (3.04.01), accessed via the CERN AFS filesystem. If you want a different db file, or
-    don't have AFS mounted, specify an explicit file path.
+    By default, the input database file will be the mass_width_2014.mcd file
+    from the most recent PDG Review of Particle Properties, which is installed
+    along with this package. If you want a different db file, or don't have AFS
+    mounted, specify an explicit file path, or set the PYPDT_DB environment variable.
     """
 
     def __init__(self, dbpath=None):
@@ -135,16 +139,15 @@ class ParticleDataTable(object):
         """
         import os
         if dbpath is None:
-            ## If no path specified, try first the local then the AFS db files
-            prefix = os.path.dirname(__file__)
-            for i in xrange(4): # remove 4 directory levels
-                prefix = os.path.dirname(prefix)
-            dbpath = os.path.join(prefix, "share", "pypdt", "particle.tbl")
-            if not os.path.exists(dbpath):
-                if os.environ.has_key("PYPDT_DB"):
-                    dbpath = os.environ["PYPDT_DB"]
-                else:
-                    dbpath = "/afs/cern.ch/sw/lcg/external/HepPDT/3.04.01/src/HepPDT-3.04.01/data/particle.tbl"
+            if os.environ.has_key("PYPDT_DB"):
+                dbpath = os.environ["PYPDT_DB"]
+            if not dbpath or not os.path.exists(dbpath):
+                ## Try the locally installed default file
+                prefix = os.path.dirname(__file__)
+                for i in xrange(4): # remove lowest 4 directory levels
+                    prefix = os.path.dirname(prefix)
+                # dbpath = os.path.join(prefix, "share", "pypdt", "particle.tbl")
+                dbpath = os.path.join(prefix, "share", "pypdt", "mass_width_2014.mcd")
         self.clear()
         self.read_db(dbpath)
 
@@ -154,23 +157,136 @@ class ParticleDataTable(object):
         self.dbfiles = []
 
     def read_db(self, dbpath):
-        """Read a particle data database file, updating the currently registered
+        """
+        Read a particle data database file, updating the currently registered
         particles. Call clear() first if you *only* want to know about the
         particle data in this db file. The db files currently in use can be
-        access via the PDT.dbfiles attribute."""
+        access via the PDT.dbfiles attribute.
+        """
+        if dbpath.endswith(".mcd"):
+            self.read_pdgmcd(dbpath)
+        elif dbpath.endswith(".csv"):
+            self.read_pdgcsv(dbpath)
+        elif dbpath.endswith(".tbl"):
+            self.read_pdttbl(dbpath)
+
+    def read_pdgmcd(self, dbpath):
+        """
+        Read a PDG .mcd particle data database file, updating the currently
+        registered particles. Call clear() first if you *only* want to know
+        about the particle data in this db file. The db files currently in use
+        can be access via the PDT.dbfiles attribute.
+        """
         self.dbfiles.append(dbpath)
         dbf = open(dbpath)
         for line in dbf:
-            ## Handle comments
-            if "//" in line:
-                line = line[:line.find("//")]
+            ## Handle comments and empty lines
+            if not line.strip() or line.startswith("*"):
+                continue
+
+            ## Parse a data line -- first block of 32 cols is up to 4 PID codes to be matched with charges list
+            data = [x.strip() for x in line[33:].split()]
+            # print data
+            ids_str = line[:32]
+            ids = [int(i) for i in ids_str.split()]
+            chgs = [c for c in data[-1].split(",")]
+            assert len(ids) == len(chgs)
+            ids_chgs = zip(ids, chgs)
+            # print ids_chgs
+
+            try:
+                mass = float(data[0])
+                width = float(data[3])
+                lifetime = None
+                basename = data[6]
+
+                for (pid, chgstr) in ids_chgs:
+                    name = basename + chgstr
+                    threecharge = None
+                    if chgstr == "0":
+                        threecharge = 0
+                    elif chgstr == "+":
+                        threecharge = 3
+                    elif chgstr == "++":
+                        threecharge = 6
+                    elif chgstr == "-":
+                        threecharge = -3
+                    elif chgstr == "--":
+                        threecharge = -6
+                    else:
+                        exec("threecharge = 3 * float(%s)" % chgstr) #< is this dangerous?
+
+                    ## Add a ParticleData object into the entries dict for this PID
+                    self.entries[pid] = ParticleData(pid, name, mass, threecharge, width, lifetime)
+                    # print name
+            except ValueError:
+                pass
+        dbf.close()
+
+    def read_pdgcsv(self, dbpath):
+        """
+        Read a PDG .csv particle data database file, updating the currently
+        registered particles. Call clear() first if you *only* want to know
+        about the particle data in this db file. The db files currently in use
+        can be access via the PDT.dbfiles attribute.
+        """
+        self.dbfiles.append(dbpath)
+        dbf = open(dbpath)
+        for line in dbf:
+            ## Handle comments and empty lines
             line = line.strip()
-            if not line:
+            if not line or line.startswith("*"):
                 continue
-            data = line.split()
-            #print data
 
             ## Parse a data line
+            data = [x.strip() for x in line.split(",")]
+            # print data
+
+            try:
+                mass = float(data[0])/1000.
+                width = float(data[3])/1000.
+                lifetime = None
+                pid = int(data[12])
+                chargestr = data[13]
+                if chargestr == "0":
+                    threecharge = 0
+                elif chargestr == "+":
+                    threecharge = 3
+                elif chargestr == "++":
+                    threecharge = 6
+                elif chargestr == "-":
+                    threecharge = -3
+                elif chargestr == "--":
+                    threecharge = -6
+                else:
+                    exec("threecharge = 3 * float(%s)" % chargestr) #< is this dangerous?
+                name = data[16]
+
+                ## Add a ParticleData object into the entries dict for this PID
+                self.entries[pid] = ParticleData(pid, name, mass, threecharge, width, lifetime)
+                # print name
+            except ValueError:
+                pass
+        dbf.close()
+
+    def read_pdttbl(self, dbpath):
+        """
+        Read a HepPDT .tbl particle data database file, updating the currently registered
+        particles. Call clear() first if you *only* want to know about the
+        particle data in this db file. The db files currently in use can be
+        access via the PDT.dbfiles attribute.
+        """
+        self.dbfiles.append(dbpath)
+        dbf = open(dbpath)
+        for line in dbf:
+            ## Handle comments and empty lines
+            line = line.strip()
+            if not line or line.startswith("//") or line.startswith("#"):
+                continue
+
+            ## Parse a data line
+            data = line.split()
+            #print data
             pid = int(data[0])
             name = data[1]
             threecharge = int(data[2])
